@@ -85,13 +85,44 @@ formula = do
 expr :: TP Exp
 expr = do
   optional (try $ symbol "\\displaystyle")
-  a <- expr1
+  (a, convertible) <- try (braces operatorname) -- needed because macros add {}
+                 <|> (expr1 >>= \e -> return (e, False))
+                 <|> operatorname
   limits <- limitsIndicator
-  subSup limits a <|> superOrSubscripted limits a <|> return a
+  subSup limits convertible a <|> superOrSubscripted limits convertible a <|> return a
+
+-- | Parser for \operatorname command.
+-- Returns a tuple of EMathOperator name and Bool depending on the flavor
+-- of the command:
+--
+--     - True for convertible operator (\operator*)
+--
+--     - False otherwise
+operatorname :: TP (Exp, Bool)
+operatorname = try $ do
+    symbol "\\operatorname"
+    convertible <- (char '*' >> spaces >> return True) <|> return False
+    op <- liftM expToOperatorName texToken
+    maybe pzero (\s -> return (EMathOperator s, convertible)) op
+
+-- | Converts identifiers, symbols and numbers to a flat string.
+-- Returns Nothing if the expression contains anything else.
+expToOperatorName :: Exp -> Maybe String
+expToOperatorName e = case e of
+            EGrouped xs ->  liftM concat $ mapM fl xs
+            _ -> fl e
+    where fl f = case f of
+                    EIdentifier s -> Just s
+                    -- handle special characters
+                    ESymbol _ "\x2212" -> Just "-"
+                    ESymbol _ "\x02B9" -> Just "'"
+                    ESymbol _ s -> Just s
+                    ENumber s -> Just s
+                    _ -> Nothing
 
 bareSubSup :: TP Exp
-bareSubSup = subSup Nothing (EIdentifier "")
-  <|> superOrSubscripted Nothing (EIdentifier "")
+bareSubSup = subSup Nothing False (EIdentifier "")
+  <|> superOrSubscripted Nothing False (EIdentifier "")
 
 limitsIndicator :: TP (Maybe Bool)
 limitsIndicator =
@@ -100,7 +131,7 @@ limitsIndicator =
   <|> return Nothing
 
 inbraces :: TP Exp
-inbraces = liftM EGrouped (braces $ many $ notFollowedBy (char '}') >> expr)
+inbraces = liftM EGrouped $ braces $ many $ notFollowedBy (char '}') >> expr
 
 texToken :: TP Exp
 texToken = texSymbol <|> inbraces <|> inbrackets <|>
@@ -127,7 +158,8 @@ left = try $ do
   symbol "\\left"
   enc <- basicEnclosure <|> (try (symbol ".") >> return (ESymbol Open "\xFEFF"))
   case enc of
-    (ESymbol Open _) -> tilRight enc <|> return (EStretchy enc)
+    (ESymbol Open _)  -> tilRight enc <|> return (EStretchy enc)
+    (ESymbol Close _) -> tilRight enc <|> return (EStretchy enc)
     _ -> pzero
 
 right :: TP Exp
@@ -136,6 +168,7 @@ right = try $ do
   enc <- basicEnclosure <|> (try (symbol ".") >> return (ESymbol Close "\xFEFF"))
   case enc of
     (ESymbol Close x) -> return (EStretchy $ ESymbol Open x)
+    (ESymbol Open x)  -> return (EStretchy $ ESymbol Open x)
     _ -> pzero
 
 -- We want stuff between \left( and \right) to be in an mrow,
@@ -165,6 +198,7 @@ endLine :: TP Char
 endLine = try $ do
   symbol "\\\\"
   optional inbrackets  -- can contain e.g. [1.0in] for a line height, not yet supported
+  optional $ try $ symbol "\\hline"   -- we don't represent the line, but it shouldn't crash parsing
   return '\n'
 
 arrayLine :: TP ArrayLine
@@ -208,7 +242,7 @@ environments = M.fromList
   , ("vmatrix", matrixWith "\x2223" "\x2223")
   , ("Vmatrix", matrixWith "\x2225" "\x2225")
   , ("split", align)
-  , ("multiline", gather)
+  , ("multline", gather)
   , ("gather", gather)
   , ("gathered", gather)
   ]
@@ -271,32 +305,32 @@ isUnderover (EUnder _ (ESymbol Accent "\xFE38")) = True  -- \underbrace
 isUnderover (EUnder _ (ESymbol Accent "\x23B5")) = True  -- \underbracket
 isUnderover _ = False
 
-subSup :: Maybe Bool -> Exp -> TP Exp
-subSup limits a = try $ do
+subSup :: Maybe Bool -> Bool -> Exp -> TP Exp
+subSup limits convertible a = try $ do
   let sub1 = symbol "_" >> expr1
   let sup1 = symbol "^" >> expr1
   (b,c) <- try (do {m <- sub1; n <- sup1; return (m,n)})
        <|> (do {n <- sup1; m <- sub1; return (m,n)})
   return $ case limits of
             Just True  -> EUnderover a b c
-            Nothing | isConvertible a -> EDownup a b c
+            Nothing | convertible || isConvertible a -> EDownup a b c
                     | isUnderover a -> EUnderover a b c
             _          -> ESubsup a b c
 
-superOrSubscripted :: Maybe Bool -> Exp -> TP Exp
-superOrSubscripted limits a = try $ do
+superOrSubscripted :: Maybe Bool -> Bool -> Exp -> TP Exp
+superOrSubscripted limits convertible a = try $ do
   c <- oneOf "^_"
   spaces
   b <- expr
   case c of
        '^' -> return $ case limits of
                         Just True  -> EOver a b
-                        Nothing | isConvertible a -> EUp a b
+                        Nothing | convertible || isConvertible a -> EUp a b
                                 | isUnderover a -> EOver a b
                         _          -> ESuper a b
        '_' -> return $ case limits of
                         Just True  -> EUnder a b
-                        Nothing | isConvertible a -> EDown a b
+                        Nothing | convertible || isConvertible a -> EDown a b
                                 | isUnderover a -> EUnder a b
                         _          -> ESub a b
        _   -> pzero
@@ -390,7 +424,7 @@ diacriticals = M.fromList
                , ("\\overleftarrow", \e -> EOver e (ESymbol Accent "\x20D6"))
                , ("\\hat", \e -> EOver e (ESymbol Accent "\x005E"))
                , ("\\widehat", \e -> EOver e (ESymbol Accent "\x0302"))
-               , ("\\tilde", \e -> EOver e (ESymbol Accent "~"))
+               , ("\\tilde", \e -> EOver e (ESymbol Accent "\x0303"))
                , ("\\widetilde", \e -> EOver e (ESymbol Accent "\x02DC"))
                , ("\\bar", \e -> EOver e (ESymbol Accent "\x203E"))
                , ("\\overbrace", \e -> EOver e (ESymbol Accent "\xFE37"))
@@ -433,11 +467,23 @@ binary = try $ do
 
 texSymbol :: TP Exp
 texSymbol = try $ do
+  negated <- (try (symbol "\\not") >> return True) <|> return False
   sym <- operator <|> command
   case M.lookup sym symbols of
-       Just s   -> return s
+       Just s   -> if negated then neg s else return s
        Nothing  -> pzero
 
+neg :: Exp -> TP Exp
+neg (ESymbol Rel x) = ESymbol Rel `fmap`
+  case x of
+       "\x2282" -> return "\x2284"
+       "\x2283" -> return "\x2285"
+       "\x2286" -> return "\x2288"
+       "\x2287" -> return "\x2289"
+       "\x2208" -> return "\x2209"
+       _        -> pzero
+neg _ = pzero
+
 -- The lexer
 lexer :: P.TokenParser st
 lexer = P.makeTokenParser texMathDef
@@ -521,11 +567,13 @@ symbols = M.fromList [
              ("+", ESymbol Bin "+")
            , ("-", ESymbol Bin "\x2212")
            , ("*", ESymbol Bin "*")
+           , ("@", ESymbol Ord "@")
            , (",", ESymbol Pun ",")
-           , (".", ESymbol Pun ".")
+           , (".", ESymbol Ord ".")
            , (";", ESymbol Pun ";")
-           , (":", ESymbol Pun ":")
-           , ("?", ESymbol Pun "?")
+           , (":", ESymbol Rel ":")
+           , ("\\colon", ESymbol Pun ":")
+           , ("?", ESymbol Ord "?")
            , (">", ESymbol Rel ">")
            , ("<", ESymbol Rel "<")
            , ("!", ESymbol Ord "!")
@@ -538,7 +586,7 @@ symbols = M.fromList [
            , ("\\mid", ESymbol Bin "\x2223")
            , ("\\parallel", ESymbol Rel "\x2225")
            , ("\\backslash", ESymbol Bin "\x2216")
-           , ("/", ESymbol Bin "/")
+           , ("/", ESymbol Ord "/")
            , ("\\setminus",	ESymbol Bin "\\")
            , ("\\times", ESymbol Bin "\x00D7")
            , ("\\alpha", EIdentifier "\x03B1")
@@ -637,6 +685,10 @@ symbols = M.fromList [
            , ("\\supset", ESymbol Rel "\x2283")
            , ("\\subseteq", ESymbol Rel "\x2286")
            , ("\\supseteq", ESymbol Rel "\x2287")
+           , ("\\nsubset", ESymbol Rel "\x2284")
+           , ("\\nsupset", ESymbol Rel "\x2285")
+           , ("\\nsubseteq", ESymbol Rel "\x2288")
+           , ("\\nsupseteq", ESymbol Rel "\x2289")
            , ("\\sqsubset", ESymbol Rel "\x228F")
            , ("\\sqsupset", ESymbol Rel "\x2290")
            , ("\\sqsubseteq", ESymbol Rel "\x2291")
